%% BYOM, with stdDEB model: byom_testdesign.m
%
% *Table of contents*

%% About
% * Author     : Tjalling Jager
% * Date       : June 2022
% * Web support: <http://www.debtox.info/byom.html>
%
% BYOM is a General framework for simulating model systems in terms of
% ordinary differential equations (ODEs). The model itself needs to be
% specified in <derivatives.html derivatives.m>, and <call_deri.html
% call_deri.m> may need to be modified to the particular problem as well.
% The files in the engine directory are needed for fitting and plotting.
% Results are shown on screen but also saved to a log file (results.out).
%
% *The model:* standard DEB model in terms of powers (energy fluxes in J/d)
% and body length. In this package, toxicant stress is included. The model
% equations are those from the 'Tromso coffee mug' ;-). Only change is that
% the ODE for structure is phrased in volumetric length rather than volume.
% The TKTD model is lifted from DEBtox2019. The 2023 publication of Jager
% et al in Ecological Modelling contains the full details:
% <https://doi.org/10.1016/j.ecolmodel.2022.110187>.
%
% *This script:* This script demonstrates the use of the parameter-space
% explorer from the openGUTS project (see <http://www.openguts.info/>) in
% making predictions for test design, given a previous calibration in a MAT
% file. This script works generically, as all species- and
% compound-specific information is loaded from the MAT file.
% 
%  Copyright (c) 2012-2023, Tjalling Jager, all rights reserved.
%  This source code is licensed under the MIT-style license found in the
%  LICENSE.txt file in the root directory of BYOM. 

%% Initial things
% Make sure that this script is in a directory somewhere *below* the BYOM
% folder.

clear, clear global % clear the workspace and globals
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
diary off           % turn of the diary function (if it is accidentaly on)
% set(0,'DefaultFigureWindowStyle','docked'); % collect all figure into one window with tab controls
set(0,'DefaultFigureWindowStyle','normal'); % separate figure windows

pathdefine(1) % set path to the BYOM/engine directory (option 1 uses parallel toolbox)
glo.basenm  = mfilename; % remember the filename for THIS file for the plots
glo.saveplt = 0; % save all plots as (1) Matlab figures, (2) JPEG file or (3) PDF (see all_options.txt)

%% The data set
% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers. The number in the top
% left of the matrix indicates how to calculate the likelihood:
%
% * -1 for multinomial likelihood (for survival data)
% * 0  for log-transform the data, then normal likelihood
% * 0.5 for square-root transform the data, then normal likelihood
% * 1  for no transformation of the data, then normal likelihood

% No data for simulations!

%% Define exposure scenario and call Matlab GUI open-file to load MAT file
% Here, a flexible scenario definition is used as demonstration. 

% NOTE: you can also load an exposure profile from a text file, using
% select_pred below. However, when loading Cw from file, modify the
% glo.LabelTable definition below as well (or comment it out).

F1 = 1;
F2 = 1;
Cw{1} = [1  1   2   3  % 1-d pulse, close interval
        0   F1*[1 2 4]
        1   0   0   0
        2   F1*[1 2 4]
        3   0   0   0
        14  F1*[1 2 4]
        15  0   0   0
        16  F1*[1 2 4]
        17  0   0   0
        25  0   0   0];

Cw{2} = [1  4   5   6 % 2-d pulse, wide interval
        0   F2*[1 2 4] 
        2   0   0   0
        7   F2*[1 2 4] 
        9   0   0   0
        14  F2*[1 2 4] 
        16  0   0   0
        25  0   0   0];     
    
make_scen(2,Cw);
% Note: I add the end of the profile in Cw here to make sure that they are
% plotted nicely, and because I define a time vector for simulation below,
% using the last time point in Cw{1}.

%% Call the Matlab GUI open-file element to load profile and MAT file

[conf_type,~,par] = select_pred([1 1 2]); 

% Note that this function, with these settings, loads a pre-saved MAT file.
% It loads the parameters, as well as all settings used to generate the MAT
% file (in glo) apart from the exposure profile (which we have already
% defined above). This only works when the MAT file was generated by BYOM
% v.6.0 BETA 7 or newer as older versions won't save glo. When using an old
% MAT file (pre BYOM v6), make sure to set glo.Tbp to the correct value if
% it needs to be >0.

%% Create custom legends for the treatments
% Tricks to make more meaningful legends for the treatments.

Scenario = [0 Cw{1}(1,2:end)]';
Label    = {'control';['close ',num2str(max(Cw{1}(2:end,2)))];
    ['close ',num2str(max(Cw{1}(2:end,3)))];
    ['close ',num2str(max(Cw{1}(2:end,4)))]};
Scenario = [Scenario;[Cw{2}(1,2:end)]'];
Label    = [Label;{['wide ',num2str(max(Cw{2}(2:end,2)))];
    ['wide ',num2str(max(Cw{2}(2:end,3)))];
    ['wide ',num2str(max(Cw{2}(2:end,4)))]}];
glo.LabelTable = table(Scenario,Label); % create a Matlab table for the labels

%% Initial values for the state variables
% Initial states, scenarios in columns, states in rows. First row are the
% 'names' of all scenarios.
% 
% Note that the complete X0mat for the calibration is also loaded with the
% MAT file. However, we need a new one for test design, so it is
% overwritten below.

X0mat = Scenario'; % the scenarios, taken from the exposure profile (here: identifiers) 
X0mat(2,:) = 0;   % initial reserve energy (J) NOT USED
X0mat(3,:) = 0;   % initial maturity level (J) NOT USED
X0mat(4,:) = 0;   % initial volumetric length (cm) NOT USED
X0mat(5,:) = 0;   % initial cumul. repro NOT USED
X0mat(6,:) = 0;   % initial values scaled damage
X0mat(7,:) = 1;   % initial values survival probability

% NOTE: the glo.locD etc. have all been loaded from the MAT file by <select_pred>.

%% Initial values for the model parameters
% All parameters have been loaded from file using select_pred. All global
% parameters, as part of the structure glo, are also loaded by select_pred
% already. Feel free to overwrite them with other values.

glo.len = 1; % switch to fit physical length (0=wwt, 1=phys. length, 2=phys. length, no shrinking) (used in call_deri.m)
% For test design, it would be useful to see shrinking if it occurs.

%% Time vector and labels for plots
% Specify what to plot. If time vector glo.t is not specified, a default is
% constructed, based on the data set.

glo.t = linspace(0,25,100); % need to define time vector as we have no data

% All plot labels have been loaded from file using select_pred.

prelim_checks % script to perform some preliminary checks and set things up
% Note: prelim_checks also fills all the options (opt_...) with defauls, so
% modify options after this call, if needed.

%% Calculations and plotting
% Here, the function is called that will do the calculation and the plotting.
% Options for the plotting can be set using opt_plot (see prelim_checks.m).
% Options for the optimsation routine can be set using opt_optim. Options
% for the ODE solver are part of the global glo. 
% 
% NOTE: for this package, the options useode and eventson in glo will not
% be functional: the ODE solver is always used, and the events function as
% well. Also note that glo.stiff and glo.break_time are loaded from the MAT
% file as well, but overwritten below. For these pulsed exposure scenarios,
% we must use glo.break_time=1, and possibly a different solver than for
% constant exposure.
 
% -------------------------------------------------------------------------
% Configurations for the ODE solver
glo.stiff = [0 3]; % ODE solver 0) ode45 (standard), 1) ode113 (moderately stiff), 2) ode15s (stiff)
% Second argument is for default sloppy (0), normally tight (1), tighter
% (2), or very tight (3) tolerances. Use 1 for quick analyses, but check
% with 3 to see if there is a difference! Especially for time-varying
% exposure, there can be large differences between the settings!
glo.break_time = 1; % break time vector up for ODE solver (1) or don't (0)
% Note: breaking the time vector is a good idea when the exposure scenario
% contains discontinuities. Don't use for continuous splines (type 1) as it
% will be much slower. For FOCUS scenarios (high time resolution), breaking
% up is not efficient and does not appear to be necessary.
% -------------------------------------------------------------------------

% % These settings for opt_optim allow you to see the parspace plot, without refitting.
% opt_optim.type     = 4; % optimisation method: 1) default simplex, 4) parspace explorer
% opt_optim.fit      = 1; % fit the parameters (1), or don't (0)
% opt_optim.ps_saved = 1; % use saved set for parameter-space explorer (1) or not (0);
% 
% opt_plot.bw        = 0; % if set to 1, plots in black and white with different plot symbols
% opt_plot.annot     = 2; % annotations in multiplot for fits: 1) box with parameter estimates 2) single legend
% opt_plot.statsup   = []; % vector with states to suppress in plotting fits
% opt_plot.limax     = 0; % if set to 1, limit axes to the data set for each stage
% 
% par_out = calc_optim(par,opt_optim); % start the optimisation
% % No need to call calc_optim unless you want to see the parspace plot.
% calc_and_plot(par_out,opt_plot); % then we can plot with the new par as well

% No need for optimisation or standard plotting here.

%% Plot results with confidence intervals
% The following code can be used to make plots with confidence intervals.
% Options for confidence bounds on model curves can be set using opt_conf
% (see prelim_checks). The plot_tktd function makes multiplots for the
% effects data, which are more readable when plotting with various
% intervals.

opt_conf.type     = conf_type; % make intervals from 1) slice sampler, 2)likelihood region, 3) parspace explorer
opt_conf.lim_set  = 1; % use limited set of n_lim points (1), or outer hull (2, not for Bayes) to create CIs
% Note: using a limited set is good for test design, but for validation,
% using the outer hull is a better idea (but will be slower).
opt_tktd.max_exp  = 0; % set to 1 to maximise exposure/damage plots on exposure rather than damage
opt_conf.set_zero = 'hb'; % parameter name to set to zero (usually the background hazard)
% Last line forces hb=0 (both in the best-fit parameter set and in the
% sample), which is a good idea when simulating, but not when validating.

% Calculate and plot dedicated TKTD plots. These plots are more readable when plotting CIs.
opt_tktd.preds   = 1; % set to 1 to only plot predictions from X0mat without data
opt_tktd.addzero = 1; % set to 1 to always add a concentration zero to X0mat

% Calculate and plot dedicated TKTD plots. These plots are more readable
% when plotting CIs.
plot_tktd([],opt_tktd,opt_conf); 
% Note that first input can be left empty: this is for the parameter
% structure, which is then obtained from the saved sample.
